@@ -16,7 +16,14 @@ You should have received a copy of the GNU General Public License
 along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#define pr_fmt(fmt)	KBUILD_MODNAME ": " fmt
+
+#include <linux/version.h>
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 37))
 #include <asm/atomic.h>
+#else
+#include <linux/atomic.h>
+#endif
 #include <linux/cdev.h>
 #include <linux/module.h>
 #include <linux/kernel.h>
@@ -24,15 +31,13 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <linux/delay.h>
 #include <linux/proc_fs.h>
 #include <linux/sched.h>
-#include <linux/version.h>
+#include <linux/vmalloc.h>
 #include <linux/wait.h>
+#include <linux/tracepoint.h>
 #include <asm/syscall.h>
 #include <net/sock.h>
-#if defined(__x86_64__)
-#include <asm/unistd_64.h>
-#else
-#include <asm/unistd_32.h>
-#endif
+
+#include <asm/unistd.h>
 
 #include "ppm_ringbuffer.h"
 #include "ppm_events_public.h"
@@ -43,8 +48,9 @@ MODULE_LICENSE("GPL");
 MODULE_AUTHOR("Draios");
 
 #define PPM_DEVICE_NAME "sysdig"
+#define PPE_DEVICE_NAME PPM_DEVICE_NAME "-events"
 
-#if (LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 35))
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35))
     #define TRACEPOINT_PROBE_REGISTER(p1, p2) tracepoint_probe_register(p1, p2)
     #define TRACEPOINT_PROBE_UNREGISTER(p1, p2) tracepoint_probe_unregister(p1, p2)
     #define TRACEPOINT_PROBE(probe, args...) static void probe(args)
@@ -65,11 +71,12 @@ struct ppm_device {
  * We have one of these for each CPU.
  */
 struct ppm_ring_buffer_context {
-	atomic_t state;
+	bool open;
+	bool capture_enabled;
 	struct ppm_ring_buffer_info *info;
 	char *buffer;
 	struct timespec last_print_time;
-	uint32_t nevents;
+	u32 nevents;
 	atomic_t preempt_count;
 	char *str_storage;	/* String storage. Size is one page. */
 };
@@ -88,6 +95,12 @@ static void record_event(enum ppm_event_type event_type,
 	struct task_struct *sched_prev,
 	struct task_struct *sched_next);
 
+static ssize_t ppe_write(struct file *filp, const char __user *buf, size_t count, loff_t *f_pos);
+
+#ifndef CONFIG_HAVE_SYSCALL_TRACEPOINTS
+ #error The kernel must have HAVE_SYSCALL_TRACEPOINTS in order for sysdig to be useful
+#endif
+
 TRACEPOINT_PROBE(syscall_enter_probe, struct pt_regs *regs, long id);
 TRACEPOINT_PROBE(syscall_exit_probe, struct pt_regs *regs, long ret);
 TRACEPOINT_PROBE(syscall_procexit_probe, struct task_struct *p);
@@ -99,6 +112,7 @@ TRACEPOINT_PROBE(sched_switch_probe, struct task_struct *prev, struct task_struc
 #endif /* (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,35)) */
 #endif /* CAPTURE_CONTEXT_SWITCHES */
 
+DECLARE_BITMAP(g_events_mask, PPM_EVENT_MAX);
 static struct ppm_device *g_ppm_devs;
 static struct class *g_ppm_class;
 static unsigned int g_ppm_numdevs;
@@ -111,17 +125,66 @@ static const struct file_operations g_ppm_fops = {
 	.owner = THIS_MODULE,
 };
 
+/* Events file operations */
+static const struct file_operations g_ppe_fops = {
+	.write = ppe_write,
+	.owner = THIS_MODULE,
+};
+
 /*
  * GLOBALS
  */
 
 static DEFINE_PER_CPU(struct ppm_ring_buffer_context*, g_ring_buffers);
-static atomic_t g_open_count;
-uint32_t g_snaplen = RW_SNAPLEN;
-uint32_t g_sampling_ratio = 1;
-static uint32_t g_sampling_interval;
+static DEFINE_MUTEX(g_open_mutex);
+static u32 g_open_count;
+u32 g_snaplen = RW_SNAPLEN;
+u32 g_sampling_ratio = 1;
+static u32 g_sampling_interval;
 static int g_is_dropping;
 static int g_dropping_mode;
+static bool g_tracepoint_registered;
+
+struct cdev *g_ppe_cdev = NULL;
+struct device *g_ppe_dev = NULL;
+
+static struct tracepoint *tp_sys_enter;
+static struct tracepoint *tp_sys_exit;
+static struct tracepoint *tp_sched_process_exit;
+#ifdef CAPTURE_CONTEXT_SWITCHES
+static struct tracepoint *tp_sched_switch;
+#endif
+
+#ifdef _DEBUG
+static bool verbose = 1;
+#else
+static bool verbose = 0;
+#endif
+
+#define vpr_info(fmt, ...)					\
+do {								\
+	if (verbose)						\
+		pr_info(fmt, ##__VA_ARGS__);			\
+} while (0)
+
+/* compat tracepoint functions */
+static int compat_register_trace(void *func, const char *probename, struct tracepoint *tp)
+{
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(3, 15, 0))
+	return TRACEPOINT_PROBE_REGISTER(probename, func);
+#else
+	return tracepoint_probe_register(tp, func, NULL);
+#endif
+}
+
+static void compat_unregister_trace(void *func, const char *probename, struct tracepoint *tp)
+{
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(3, 15, 0))
+	TRACEPOINT_PROBE_UNREGISTER(probename, func);
+#else
+	tracepoint_probe_unregister(tp, func, NULL);
+#endif
+}
 
 /*
  * user I/O functions
@@ -132,20 +195,31 @@ static int ppm_open(struct inode *inode, struct file *filp)
 	struct ppm_ring_buffer_context *ring;
 	int ring_no = iminor(filp->f_dentry->d_inode);
 
-	trace_enter();
+	mutex_lock(&g_open_mutex);
 
 	ring = per_cpu(g_ring_buffers, ring_no);
 
-	if (atomic_cmpxchg(&ring->state, CS_STOPPED, CS_STARTED) != CS_STOPPED) {
-		pr_info("sysdig-probe: invalid operation: attempting to open device %d multiple times\n", ring_no);
-		return -EBUSY;
+	if (ring->open) {
+		pr_err("invalid operation: attempting to open device %d multiple times\n", ring_no);
+		ret = -EBUSY;
+		goto cleanup_open;
 	}
 
+	vpr_info("opening ring %d\n", ring_no);
+
+	/*
+	 * ring->preempt_count is not reset to 0 on purpose, to prevent a race condition:
+	 * if the same device is quickly closed and then reopened, record_event() might still be executing 
+	 * (with ring->preempt_count to 1) while ppm_open() resets ring->preempt_count to 0. 
+	 * When record_event() will exit, it will decrease 
+	 * ring->preempt_count which will become < 0, leading to the complete loss of all the events for that CPU.
+	 */
 	g_dropping_mode = 0;
 	g_snaplen = RW_SNAPLEN;
 	g_sampling_ratio = 1;
 	g_sampling_interval = 0;
 	g_is_dropping = 0;
+	bitmap_fill(g_events_mask, PPM_EVENT_MAX); /* Enable all syscall to be passed to userspace */
 	ring->info->head = 0;
 	ring->info->tail = 0;
 	ring->nevents = 0;
@@ -154,92 +228,81 @@ static int ppm_open(struct inode *inode, struct file *filp)
 	ring->info->n_drops_pf = 0;
 	ring->info->n_preemptions = 0;
 	ring->info->n_context_switches = 0;
-	atomic_set(&ring->preempt_count, 0);
+	ring->capture_enabled = true;
 	getnstimeofday(&ring->last_print_time);
+	ring->open = true;
 
-	/*
-	 * The last open device starts the collection
-	 */
-	if (atomic_inc_return(&g_open_count) == g_ppm_numdevs) {
-		/* struct task_struct *task; */
-
-		pr_info("sysdig-probe: starting capture\n");
-
-		/*
-				//
-				// Before enabling the tracepoints, we add the current list of running processes as events in buffer 0
-				//
-				for_each_process(task) {
-					printk("%s [%d]\n",task->comm , task->pid);
-				}
-		*/
-
+	if (!g_tracepoint_registered) {
+		vpr_info("starting capture\n");
 		/*
 		 * Enable the tracepoints
 		 */
-		ret = TRACEPOINT_PROBE_REGISTER("sys_exit", (void *) syscall_exit_probe);
+		ret = compat_register_trace(syscall_exit_probe, "sys_exit", tp_sys_exit);
 		if (ret) {
-			pr_err("sysdig-probe: can't create the sys_exit tracepoint\n");
-			return ret;
+			pr_err("can't create the sys_exit tracepoint\n");
+			goto err_sys_exit;
 		}
 
-		ret = TRACEPOINT_PROBE_REGISTER("sys_enter", (void *) syscall_enter_probe);
+		ret = compat_register_trace(syscall_enter_probe, "sys_enter", tp_sys_enter);
 		if (ret) {
-			TRACEPOINT_PROBE_UNREGISTER("sys_exit",
-						    (void *) syscall_exit_probe);
-
-			pr_err("sysdig-probe: can't create the sys_enter tracepoint\n");
-
-			return ret;
+			pr_err("can't create the sys_enter tracepoint\n");
+			goto err_sys_enter;
 		}
 
-		ret = TRACEPOINT_PROBE_REGISTER("sched_process_exit", (void *) syscall_procexit_probe);
+		ret = compat_register_trace(syscall_procexit_probe, "sched_process_exit", tp_sched_process_exit);
 		if (ret) {
-			TRACEPOINT_PROBE_UNREGISTER("sys_exit",
-						    (void *) syscall_exit_probe);
-			TRACEPOINT_PROBE_UNREGISTER("sys_enter",
-						    (void *) syscall_enter_probe);
-
-			pr_err("sysdig-probe: can't create the sched_process_exit tracepoint\n");
-
-			return ret;
+			pr_err("can't create the sched_process_exit tracepoint\n");
+			goto err_sched_procexit;
 		}
 
 #ifdef CAPTURE_CONTEXT_SWITCHES
-		ret = TRACEPOINT_PROBE_REGISTER("sched_switch", (void *) sched_switch_probe);
+		ret = compat_register_trace(sched_switch_probe, "sched_switch", tp_sched_switch);
 		if (ret) {
-			TRACEPOINT_PROBE_UNREGISTER("sys_exit",
-						    (void *) syscall_exit_probe);
-			TRACEPOINT_PROBE_UNREGISTER("sys_enter",
-						    (void *) syscall_enter_probe);
-			TRACEPOINT_PROBE_UNREGISTER("sched_process_exit",
-						    (void *) syscall_procexit_probe);
-
-			pr_err("sysdig-probe: can't create the sched_switch tracepoint\n");
-
-			return ret;
+			pr_err("can't create the sched_switch tracepoint\n");
+			goto err_sched_switch;
 		}
 #endif
+		g_tracepoint_registered = true;
 	}
 
-	return 0;
+	++g_open_count;
+	ret = 0;
+	
+	goto cleanup_open;
+
+err_sched_switch:
+	compat_unregister_trace(syscall_procexit_probe, "sched_process_exit", tp_sched_process_exit);
+err_sched_procexit:
+	compat_unregister_trace(syscall_enter_probe, "sys_enter", tp_sys_enter);
+err_sys_enter:
+	compat_unregister_trace(syscall_exit_probe, "sys_exit", tp_sys_exit);
+err_sys_exit:
+	ring->open = false;
+cleanup_open:
+	mutex_unlock(&g_open_mutex);
+
+	return ret;
 }
 
 static int ppm_release(struct inode *inode, struct file *filp)
 {
+	int ret;
 	struct ppm_ring_buffer_context *ring;
 	int ring_no = iminor(filp->f_dentry->d_inode);
 
-	trace_enter();
+	mutex_lock(&g_open_mutex);
 
 	ring = per_cpu(g_ring_buffers, ring_no);
 
-	if (atomic_xchg(&ring->state, CS_STOPPED) == CS_STOPPED) {
-		pr_info("sysdig-probe: attempting to close unopened device %d\n", ring_no);
-		return -EBUSY;
+	if (!ring->open) {
+		pr_err("attempting to close unopened device %d\n", ring_no);
+		ret = -EBUSY;
+		goto cleanup_release;
 	}
 
-	pr_info("sysdig-probe: closing ring %d, evt:%llu, dr_buf:%llu, dr_pf:%llu, pr:%llu, cs:%llu\n",
+	ring->capture_enabled = false;
+
+	vpr_info("closing ring %d, evt:%llu, dr_buf:%llu, dr_pf:%llu, pr:%llu, cs:%llu\n",
 	       ring_no,
 	       ring->info->n_evts,
 	       ring->info->n_drops_buffer,
@@ -250,24 +313,32 @@ static int ppm_release(struct inode *inode, struct file *filp)
 	/*
 	 * The last closed device stops event collection
 	 */
-	if (atomic_dec_return(&g_open_count) == 0) {
-		pr_info("sysdig-probe: stopping capture\n");
+	--g_open_count;
+	if (g_open_count == 0) {
+		if (g_tracepoint_registered) {
+			vpr_info("stopping capture\n");
 
-		TRACEPOINT_PROBE_UNREGISTER("sys_exit",
-					    (void *) syscall_exit_probe);
+			compat_unregister_trace(syscall_exit_probe, "sys_exit", tp_sys_exit);
+			compat_unregister_trace(syscall_enter_probe, "sys_enter", tp_sys_enter);
+			compat_unregister_trace(syscall_procexit_probe, "sched_process_exit", tp_sched_process_exit);
 
-		TRACEPOINT_PROBE_UNREGISTER("sys_enter",
-					    (void *) syscall_enter_probe);
-
-		TRACEPOINT_PROBE_UNREGISTER("sched_process_exit",
-					    (void *) syscall_procexit_probe);
 #ifdef CAPTURE_CONTEXT_SWITCHES
-		TRACEPOINT_PROBE_UNREGISTER("sched_switch",
-					    (void *) sched_switch_probe);
+			compat_unregister_trace(sched_switch_probe, "sched_switch", tp_sched_switch);
 #endif
+			tracepoint_synchronize_unregister();
+			g_tracepoint_registered = false;
+		} else {
+			ASSERT(false);
+		}
 	}
 
-	return 0;
+	ring->open = false;
+	ret = 0;
+
+cleanup_release:
+	mutex_unlock(&g_open_mutex);
+
+	return ret;
 }
 
 static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
@@ -278,9 +349,11 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 		int ring_no = iminor(filp->f_dentry->d_inode);
 		struct ppm_ring_buffer_context *ring = per_cpu(g_ring_buffers, ring_no);
 
-		atomic_set(&(ring->state), CS_INACTIVE);
+		mutex_lock(&g_open_mutex);
+		ring->capture_enabled = false;
+		mutex_unlock(&g_open_mutex);
 
-		pr_info("sysdig-probe: PPM_IOCTL_DISABLE_CAPTURE for ring %d\n", ring_no);
+		vpr_info("PPM_IOCTL_DISABLE_CAPTURE for ring %d\n", ring_no);
 
 		return 0;
 	}
@@ -289,28 +362,30 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 		int ring_no = iminor(filp->f_dentry->d_inode);
 		struct ppm_ring_buffer_context *ring = per_cpu(g_ring_buffers, ring_no);
 
-		atomic_set(&ring->state, CS_STARTED);
+		mutex_lock(&g_open_mutex);
+		ring->capture_enabled = true;
+		mutex_unlock(&g_open_mutex);
 
-		pr_info("sysdig-probe: PPM_IOCTL_ENABLE_CAPTURE for ring %d\n", ring_no);
+		vpr_info("PPM_IOCTL_ENABLE_CAPTURE for ring %d\n", ring_no);
 
 		return 0;
 	}
 	case PPM_IOCTL_DISABLE_DROPPING_MODE:
 	{
 		g_dropping_mode = 0;
-		pr_info("sysdig-probe: PPM_IOCTL_DISABLE_DROPPING_MODE\n");
+		vpr_info("PPM_IOCTL_DISABLE_DROPPING_MODE\n");
 		g_sampling_interval = 1000000000;
 		g_sampling_ratio = 1;
 		return 0;
 	}
 	case PPM_IOCTL_ENABLE_DROPPING_MODE:
 	{
-		uint32_t new_sampling_ratio;
+		u32 new_sampling_ratio;
 
 		g_dropping_mode = 1;
-		pr_info("sysdig-probe: PPM_IOCTL_ENABLE_DROPPING_MODE\n");
+		vpr_info("PPM_IOCTL_ENABLE_DROPPING_MODE\n");
 
-		new_sampling_ratio = (uint32_t)arg;
+		new_sampling_ratio = (u32)arg;
 
 		if (new_sampling_ratio != 1 &&
 			new_sampling_ratio != 2 &&
@@ -320,33 +395,76 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 			new_sampling_ratio != 32 &&
 			new_sampling_ratio != 64 &&
 			new_sampling_ratio != 128) {
-			pr_info("sysdig-probe: invalid sampling ratio %u\n", new_sampling_ratio);
+			pr_err("invalid sampling ratio %u\n", new_sampling_ratio);
 			return -EINVAL;
 		}
 
 		g_sampling_interval = 1000000000 / new_sampling_ratio;
 		g_sampling_ratio = new_sampling_ratio;
 
-		pr_info("sysdig-probe: new sampling ratio: %d\n", new_sampling_ratio);
+		vpr_info("new sampling ratio: %d\n", new_sampling_ratio);
 		return 0;
 	}
 	case PPM_IOCTL_SET_SNAPLEN:
 	{
-		uint32_t new_snaplen;
+		u32 new_snaplen;
 
-		pr_info("sysdig-probe: PPM_IOCTL_SET_SNAPLEN\n");
-		new_snaplen = (uint32_t)arg;
+		vpr_info("PPM_IOCTL_SET_SNAPLEN\n");
+		new_snaplen = (u32)arg;
 
 		if (new_snaplen > RW_MAX_SNAPLEN) {
-			pr_info("sysdig-probe: invalid snaplen %u\n", new_snaplen);
+			pr_err("invalid snaplen %u\n", new_snaplen);
 			return -EINVAL;
 		}
 
 		g_snaplen = new_snaplen;
 
-		pr_info("sysdig-probe: new snaplen: %d\n", g_snaplen);
+		vpr_info("new snaplen: %d\n", g_snaplen);
 		return 0;
 	}
+
+	case PPM_IOCTL_MASK_ZERO_EVENTS:
+	{
+		vpr_info("PPM_IOCTL_MASK_ZERO_EVENTS\n");
+
+		bitmap_zero(g_events_mask, PPM_EVENT_MAX);
+
+		/* Used for dropping events so they must stay on */
+		set_bit(PPME_DROP_E, g_events_mask);
+		set_bit(PPME_DROP_X, g_events_mask);
+		return 0;
+	}
+
+	case PPM_IOCTL_MASK_SET_EVENT:
+	{
+		u32 syscall_to_set = (u32)arg;
+
+		vpr_info("PPM_IOCTL_MASK_SET_EVENT (%u)\n", syscall_to_set);
+
+		if (syscall_to_set > PPM_EVENT_MAX) {
+			pr_err("invalid syscall %u\n", syscall_to_set);
+			return -EINVAL;
+		}
+
+		set_bit(syscall_to_set, g_events_mask);
+		return 0;
+	}
+
+	case PPM_IOCTL_MASK_UNSET_EVENT:
+	{
+		u32 syscall_to_unset = (u32)arg;
+
+		vpr_info("PPM_IOCTL_MASK_UNSET_EVENT (%u)\n", syscall_to_unset);
+
+		if (syscall_to_unset > NR_syscalls) {
+			pr_err("invalid syscall %u\n", syscall_to_unset);
+			return -EINVAL;
+		}
+
+		clear_bit(syscall_to_unset, g_events_mask);
+		return 0;
+	}
+
 	default:
 		return -ENOTTY;
 	}
@@ -354,8 +472,6 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 
 static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 {
-	trace_enter();
-
 	if (vma->vm_pgoff == 0) {
 		int ret;
 		long length = vma->vm_end - vma->vm_start;
@@ -366,7 +482,7 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 		int ring_no = iminor(filp->f_dentry->d_inode);
 		struct ppm_ring_buffer_context *ring;
 
-		pr_info("sysdig-probe: mmap for CPU %d, start=%lu len=%ld page_size=%lu\n",
+		vpr_info("mmap for CPU %d, start=%lu len=%ld page_size=%lu\n",
 		       ring_no,
 		       useraddr,
 		       length,
@@ -376,14 +492,14 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 		 * Enforce ring buffer size
 		 */
 		if (RING_BUF_SIZE < 2 * PAGE_SIZE) {
-			pr_info("sysdig-probe: Ring buffer size too small (%ld bytes, must be at least %ld bytes\n",
+			pr_err("Ring buffer size too small (%ld bytes, must be at least %ld bytes\n",
 			       (long)RING_BUF_SIZE,
 			       (long)PAGE_SIZE);
 			return -EIO;
 		}
 
 		if (RING_BUF_SIZE / PAGE_SIZE * PAGE_SIZE != RING_BUF_SIZE) {
-			pr_info("sysdig-probe: Ring buffer size is not a multiple of the page size\n");
+			pr_err("Ring buffer size is not a multiple of the page size\n");
 			return -EIO;
 		}
 
@@ -397,7 +513,7 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 			 * When the size requested by the user is smaller than a page, we assume
 			 * she's mapping the ring info structure
 			 */
-			pr_info("sysdig-probe: mapping the ring info\n");
+			vpr_info("mapping the ring info\n");
 
 			vmalloc_area_ptr = (char *)ring->info;
 			orig_vmalloc_area_ptr = vmalloc_area_ptr;
@@ -407,7 +523,7 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 			ret = remap_pfn_range(vma, useraddr, pfn,
 					      PAGE_SIZE, PAGE_SHARED);
 			if (ret < 0) {
-				pr_info("sysdig-probe: remap_pfn_range failed (1)\n");
+				pr_err("remap_pfn_range failed (1)\n");
 				return ret;
 			}
 
@@ -419,7 +535,7 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 			 * When the size requested by the user equals the ring buffer size, we map the full
 			 * buffer
 			 */
-			pr_info("sysdig-probe: mapping the data buffer\n");
+			vpr_info("mapping the data buffer\n");
 
 			vmalloc_area_ptr = (char *)ring->buffer;
 			orig_vmalloc_area_ptr = vmalloc_area_ptr;
@@ -427,8 +543,8 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 			/*
 			 * Validate that the buffer access is read only
 			 */
-			if (vma->vm_flags & (VM_WRITE | VM_EXEC)) {
-				pr_info("sysdig-probe: invalid mmap flags 0x%lx\n", vma->vm_flags);
+			if (vma->vm_flags & VM_WRITE) {
+				pr_err("invalid mmap flags 0x%lx\n", vma->vm_flags);
 				return -EIO;
 			}
 
@@ -443,7 +559,7 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 				ret = remap_pfn_range(vma, useraddr, pfn,
 						      PAGE_SIZE, PAGE_SHARED);
 				if (ret < 0) {
-					pr_info("sysdig-probe: remap_pfn_range failed (1)\n");
+					pr_err("remap_pfn_range failed (1)\n");
 					return ret;
 				}
 
@@ -465,7 +581,7 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 				ret = remap_pfn_range(vma, useraddr, pfn,
 						      PAGE_SIZE, PAGE_SHARED);
 				if (ret < 0) {
-					pr_info("sysdig-probe: remap_pfn_range failed (1)\n");
+					pr_err("remap_pfn_range failed (1)\n");
 					return ret;
 				}
 
@@ -476,15 +592,20 @@ static int ppm_mmap(struct file *filp, struct vm_area_struct *vma)
 
 			return 0;
 		} else {
-			pr_info("sysdig-probe: Invalid mmap size %ld\n", length);
+			pr_err("Invalid mmap size %ld\n", length);
 			return -EIO;
 		}
 	}
 
-	pr_info("sysdig-probe: invalid pgoff %lu, must be 0\n", vma->vm_pgoff);
+	pr_err("invalid pgoff %lu, must be 0\n", vma->vm_pgoff);
 	return -EIO;
 }
 
+static ssize_t ppe_write(struct file *filp, const char __user *buf, size_t count, loff_t *f_pos)
+{
+	return count;
+}
+
 /* Argument list sizes for sys_socketcall */
 #define AL(x) ((x) * sizeof(unsigned long))
 static const unsigned char nas[21] = {
@@ -495,7 +616,7 @@ static const unsigned char nas[21] = {
 };
 #undef AL
 
-#ifndef __x86_64__
+#ifdef __NR_socketcall
 static enum ppm_event_type parse_socketcall(struct event_filler_arguments *filler_args, struct pt_regs *regs)
 {
 	unsigned long __user args[2];
@@ -569,7 +690,7 @@ static enum ppm_event_type parse_socketcall(struct event_filler_arguments *fille
 		return PPME_GENERIC_E;
 	}
 }
-#endif /* __x86_64__ */
+#endif /* __NR_socketcall */
 
 static inline int drop_event(enum ppm_event_type event_type, int never_drop, struct timespec *ts)
 {
@@ -604,30 +725,36 @@ static void record_event(enum ppm_event_type event_type,
 {
 	size_t event_size;
 	int next;
-	uint32_t freespace;
-	uint32_t usedspace;
+	u32 freespace;
+	u32 usedspace;
 	struct event_filler_arguments args;
-	uint32_t ttail;
-	uint32_t head;
+	u32 ttail;
+	u32 head;
 	struct ppm_ring_buffer_context *ring;
 	struct ppm_ring_buffer_info *ring_info;
 	int drop = 1;
 	int32_t cbres = PPM_SUCCESS;
 	struct timespec ts;
 
-	trace_enter();
-
 	getnstimeofday(&ts);
 
+	if (!test_bit(event_type, g_events_mask))
+		return;
+
 	if (drop_event(event_type, never_drop, &ts))
 		return;
 
-	ring = get_cpu_var(g_ring_buffers);
-	ring_info = ring->info;
-
 	/*
 	 * FROM THIS MOMENT ON, WE HAVE TO BE SUPER FAST
 	 */
+	ring = get_cpu_var(g_ring_buffers);
+	ring_info = ring->info;
+
+	if (!ring->capture_enabled) {
+		put_cpu_var(g_ring_buffers);
+		return;
+	}
+
 	ring_info->n_evts++;
 	if (sched_prev != NULL) {
 		ASSERT(sched_prev != NULL);
@@ -647,12 +774,6 @@ static void record_event(enum ppm_event_type event_type,
 		return;
 	}
 
-	if (unlikely(atomic_read(&ring->state) == CS_INACTIVE)) {
-		atomic_dec(&ring->preempt_count);
-		put_cpu_var(g_ring_buffers);
-		return;
-	}
-
 	/*
 	 * Calculate the space currently available in the buffer
 	 */
@@ -671,7 +792,7 @@ static void record_event(enum ppm_event_type event_type,
 	ASSERT(ttail <= RING_BUF_SIZE);
 	ASSERT(head <= RING_BUF_SIZE);
 
-#ifndef __x86_64__
+#ifdef __NR_socketcall
 	/*
 	 * If this is a socketcall system call, determine the correct event type
 	 * by parsing the arguments and patch event_type accordingly
@@ -684,6 +805,7 @@ static void record_event(enum ppm_event_type event_type,
 	 */
 	if (regs && id == __NR_socketcall) {
 		enum ppm_event_type tet;
+
 		tet = parse_socketcall(&args, regs);
 
 		if (event_type == PPME_GENERIC_E)
@@ -699,7 +821,7 @@ static void record_event(enum ppm_event_type event_type,
 	 * Determine how many arguments this event has
 	 */
 	args.nargs = g_event_info[event_type].nparams;
-	args.arg_data_offset = args.nargs * sizeof(uint16_t);
+	args.arg_data_offset = args.nargs * sizeof(u16);
 
 	/*
 	 * Make sure we have enough space for the event header.
@@ -725,7 +847,7 @@ static void record_event(enum ppm_event_type event_type,
 #ifdef PPM_ENABLE_SENTINEL
 		args.sentinel = ring->nevents;
 #endif
-		args.buffer_size = min(freespace, (uint32_t)(2 * PAGE_SIZE)) - sizeof(struct ppm_evt_hdr); /* freespace is guaranteed to be bigger than sizeof(struct ppm_evt_hdr) */
+		args.buffer_size = min(freespace, (u32)(2 * PAGE_SIZE)) - sizeof(struct ppm_evt_hdr); /* freespace is guaranteed to be bigger than sizeof(struct ppm_evt_hdr) */
 		args.event_type = event_type;
 		args.regs = regs;
 		args.sched_prev = sched_prev;
@@ -760,7 +882,7 @@ static void record_event(enum ppm_event_type event_type,
 				hdr->len = event_size;
 				drop = 0;
 			} else {
-				pr_info("sysdig-probe: corrupted filler for event type %d (added %u args, should have added %u)\n",
+				pr_err("corrupted filler for event type %d (added %u args, should have added %u)\n",
 				       event_type,
 				       args.curarg,
 				       args.nargs);
@@ -804,7 +926,7 @@ static void record_event(enum ppm_event_type event_type,
 			ring_info->n_drops_buffer++;
 		} else if (cbres == PPM_FAILURE_INVALID_USER_MEMORY) {
 #ifdef _DEBUG
-			pr_info("sysdig-probe: Invalid read from user for event %d\n", event_type);
+			pr_err("Invalid read from user for event %d\n", event_type);
 #endif
 			ring_info->n_drops_pf++;
 		} else if (cbres == PPM_FAILURE_BUFFER_FULL) {
@@ -816,7 +938,7 @@ static void record_event(enum ppm_event_type event_type,
 
 #ifdef _DEBUG
 	if (ts.tv_sec > ring->last_print_time.tv_sec + 1) {
-		pr_info("sysdig-probe: CPU%d, use:%d%%, ev:%llu, dr_buf:%llu, dr_pf:%llu, pr:%llu, cs:%llu\n",
+		vpr_info("CPU%d, use:%d%%, ev:%llu, dr_buf:%llu, dr_pf:%llu, pr:%llu, cs:%llu\n",
 		       smp_processor_id(),
 		       (usedspace * 100) / RING_BUF_SIZE,
 		       ring_info->n_evts,
@@ -837,8 +959,9 @@ static void record_event(enum ppm_event_type event_type,
 
 TRACEPOINT_PROBE(syscall_enter_probe, struct pt_regs *regs, long id)
 {
-	trace_enter();
+	long table_index;
 
+#ifdef CONFIG_X86_64
 	/*
 	 * If this is a 32bit process running on a 64bit kernel (see the CONFIG_IA32_EMULATION
 	 * kernel flag), we skip its events.
@@ -846,13 +969,15 @@ TRACEPOINT_PROBE(syscall_enter_probe, struct pt_regs *regs, long id)
 	 */
 	if (unlikely(test_tsk_thread_flag(current, TIF_IA32)))
 		return;
+#endif
 
-	if (likely(id >= 0 && id < SYSCALL_TABLE_SIZE)) {
-		int used = g_syscall_table[id].flags & UF_USED;
-		int never_drop = g_syscall_table[id].flags & UF_NEVER_DROP;
+	table_index = id - SYSCALL_TABLE_ID0;
+	if (likely(table_index >= 0 && table_index < SYSCALL_TABLE_SIZE)) {
+		int used = g_syscall_table[table_index].flags & UF_USED;
+		int never_drop = g_syscall_table[table_index].flags & UF_NEVER_DROP;
 
 		if (used)
-			record_event(g_syscall_table[id].enter_event_type, regs, id, never_drop, NULL, NULL);
+			record_event(g_syscall_table[table_index].enter_event_type, regs, id, never_drop, NULL, NULL);
 		else
 			record_event(PPME_GENERIC_E, regs, id, false, NULL, NULL);
 	}
@@ -861,9 +986,9 @@ TRACEPOINT_PROBE(syscall_enter_probe, struct pt_regs *regs, long id)
 TRACEPOINT_PROBE(syscall_exit_probe, struct pt_regs *regs, long ret)
 {
 	int id;
+	long table_index;
 
-	trace_enter();
-
+#ifdef CONFIG_X86_64
 	/*
      * If this is a 32bit process running on a 64bit kernel (see the CONFIG_IA32_EMULATION
 	 * kernel flag), we skip its events.
@@ -871,15 +996,17 @@ TRACEPOINT_PROBE(syscall_exit_probe, struct pt_regs *regs, long ret)
 	 */
 	if (unlikely(test_tsk_thread_flag(current, TIF_IA32)))
 		return;
+#endif
 
 	id = syscall_get_nr(current, regs);
 
-	if (likely(id >= 0 && id < SYSCALL_TABLE_SIZE)) {
-		int used = g_syscall_table[id].flags & UF_USED;
-		int never_drop = g_syscall_table[id].flags & UF_NEVER_DROP;
+	table_index = id - SYSCALL_TABLE_ID0;
+	if (likely(table_index >= 0 && table_index < SYSCALL_TABLE_SIZE)) {
+		int used = g_syscall_table[table_index].flags & UF_USED;
+		int never_drop = g_syscall_table[table_index].flags & UF_NEVER_DROP;
 
 		if (used)
-			record_event(g_syscall_table[id].exit_event_type, regs, id, never_drop, NULL, NULL);
+			record_event(g_syscall_table[table_index].exit_event_type, regs, id, never_drop, NULL, NULL);
 		else
 			record_event(PPME_GENERIC_X, regs, id, false, NULL, NULL);
 	}
@@ -890,8 +1017,6 @@ int __access_remote_vm(struct task_struct *t, struct mm_struct *mm, unsigned lon
 
 TRACEPOINT_PROBE(syscall_procexit_probe, struct task_struct *p)
 {
-	trace_enter();
-
 	if (unlikely(current->flags & PF_KTHREAD)) {
 		/*
 		 * We are not interested in kernel threads
@@ -913,7 +1038,7 @@ TRACEPOINT_PROBE(sched_switch_probe, struct rq *rq, struct task_struct *prev, st
 TRACEPOINT_PROBE(sched_switch_probe, struct task_struct *prev, struct task_struct *next)
 #endif
 {
-	record_event(PPME_SCHEDSWITCH_E,
+	record_event(PPME_SCHEDSWITCH_6_E,
 		NULL,
 		-1,
 		0,
@@ -925,14 +1050,13 @@ TRACEPOINT_PROBE(sched_switch_probe, struct task_struct *prev, struct task_struc
 static struct ppm_ring_buffer_context *alloc_ring_buffer(struct ppm_ring_buffer_context **ring)
 {
 	unsigned int j;
-	trace_enter();
 
 	/*
 	 * Allocate the ring descriptor
 	 */
 	*ring = vmalloc(sizeof(struct ppm_ring_buffer_context));
 	if (*ring == NULL) {
-		pr_err("sysdig-probe: Error allocating ring memory\n");
+		pr_err("Error allocating ring memory\n");
 		return NULL;
 	}
 
@@ -941,9 +1065,8 @@ static struct ppm_ring_buffer_context *alloc_ring_buffer(struct ppm_ring_buffer_
 	 */
 	(*ring)->str_storage = (char *)__get_free_page(GFP_USER);
 	if (!(*ring)->str_storage) {
-		pr_err("sysdig-probe: Error allocating the string storage\n");
-		vfree(*ring);
-		return NULL;
+		pr_err("Error allocating the string storage\n");
+		goto err_str_storage;
 	}
 
 	/*
@@ -953,10 +1076,8 @@ static struct ppm_ring_buffer_context *alloc_ring_buffer(struct ppm_ring_buffer_
 	 */
 	(*ring)->buffer = vmalloc(RING_BUF_SIZE + 2 * PAGE_SIZE);
 	if ((*ring)->buffer == NULL) {
-		pr_err("sysdig-probe: Error allocating ring memory\n");
-		free_page((unsigned long)(*ring)->str_storage);
-		vfree(*ring);
-		return NULL;
+		pr_err("Error allocating ring memory\n");
+		goto err_buffer;
 	}
 
 	for (j = 0; j < RING_BUF_SIZE + 2 * PAGE_SIZE; j++)
@@ -967,22 +1088,15 @@ static struct ppm_ring_buffer_context *alloc_ring_buffer(struct ppm_ring_buffer_
 	 */
 	(*ring)->info = vmalloc(sizeof(struct ppm_ring_buffer_info));
 	if ((*ring)->info == NULL) {
-		pr_err("sysdig-probe: Error allocating ring memory\n");
-		vfree((void *)(*ring)->buffer);
-		free_page((unsigned long)(*ring)->str_storage);
-		vfree(*ring);
-		return NULL;
+		pr_err("Error allocating ring memory\n");
+		goto err_ring_info;
 	}
 
-/* for(j = 0; j < (RING_BUF_SIZE / PAGE_SIZE + 1); j += PAGE_SIZE) */
-/* { */
-/* SetPageReserved(vmalloc_to_page(ring) + j); */
-/* } */
-
 	/*
 	 * Initialize the buffer info structure
 	 */
-	atomic_set(&(*ring)->state, CS_STOPPED);
+	(*ring)->open = false;
+	(*ring)->capture_enabled = false;
 	(*ring)->info->head = 0;
 	(*ring)->info->tail = 0;
 	(*ring)->nevents = 0;
@@ -994,45 +1108,93 @@ static struct ppm_ring_buffer_context *alloc_ring_buffer(struct ppm_ring_buffer_
 	atomic_set(&(*ring)->preempt_count, 0);
 	getnstimeofday(&(*ring)->last_print_time);
 
-	pr_info("sysdig-probe: CPU buffer initialized, size=%d\n", RING_BUF_SIZE);
+	pr_info("CPU buffer initialized, size=%d\n", RING_BUF_SIZE);
 
 	return *ring;
+
+err_ring_info:
+	vfree((void *)(*ring)->buffer);
+err_buffer:
+	free_page((unsigned long)(*ring)->str_storage);
+err_str_storage:
+	vfree(*ring);
+
+	return NULL;
 }
 
 static void free_ring_buffer(struct ppm_ring_buffer_context *ring)
 {
-	trace_enter();
-
 	vfree(ring->info);
 	vfree((void *)ring->buffer);
 	free_page((unsigned long)ring->str_storage);
 	vfree(ring);
 }
 
-/* static int ppm_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data) */
-/* { */
-/* int len = 0; */
-/* int j; */
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 15, 0))
+static void visit_tracepoint(struct tracepoint *tp, void *priv)
+{
+	if (!strcmp(tp->name, "sys_enter"))
+		tp_sys_enter = tp;
+	else if (!strcmp(tp->name, "sys_exit"))
+		tp_sys_exit = tp;
+	else if (!strcmp(tp->name, "sched_process_exit"))
+		tp_sched_process_exit = tp;
+#ifdef CAPTURE_CONTEXT_SWITCHES
+	else if (!strcmp(tp->name, "sched_switch"))
+		tp_sched_switch = tp;
+#endif
+}
 
-/* trace_enter(); */
+static int get_tracepoint_handles(void)
+{
+	for_each_kernel_tracepoint(visit_tracepoint, NULL);
 
-/* for(j = 0; j < NR_syscalls; ++j) */
-/* { */
-/* #if defined(__x86_64__) */
-/* len += snprintf(page + len, count - len, "%ld\t%ld\n", */
-/* atomic64_read(&g_syscall_count[j].count), */
-/* atomic64_read(&g_syscall_count[j].count) ? (atomic64_read(&g_syscall_count[j].tot_time_ns) / atomic64_read(&g_syscall_count[j].count)) : 0); */
-/* #else */
-/* len += snprintf(page + len, count - len, "%lld\n", */
-/* atomic64_read(&g_syscall_count[j].count)); */
-/* #endif */
-/* } */
+	if (!tp_sys_enter) {
+		pr_err("failed to find sys_enter tracepoint\n");
+		return -ENOENT;
+	}
+	if (!tp_sys_exit) {
+		pr_err("failed to find sys_exit tracepoint\n");
+		return -ENOENT;
+	}
+	if (!tp_sched_process_exit) {
+		pr_err("failed to find sched_process_exit tracepoint\n");
+		return -ENOENT;
+	}
+#ifdef CAPTURE_CONTEXT_SWITCHES
+	if (!tp_sched_switch) {
+		pr_err("failed to find sched_switch tracepoint\n");
+		return -ENOENT;
+	}
+#endif
 
-/* *eof = 1; */
-/* return len; */
-/* } */
+	return 0;
+}
+#else
+static int get_tracepoint_handles(void)
+{
+	return 0;
+}
+#endif
 
-int init_module(void)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 3, 0)
+static char *ppm_devnode(struct device *dev, umode_t *mode)
+#else
+static char *ppm_devnode(struct device *dev, mode_t *mode)
+#endif
+{
+	if (mode) {
+		*mode = 0400;
+
+		if (dev)
+			if (MINOR(dev->devt) == g_ppm_numdevs)
+				*mode = 0222;
+	}
+
+	return NULL;
+}
+
+int sysdig_init(void)
 {
 	dev_t dev;
 	unsigned int cpu;
@@ -1043,7 +1205,11 @@ int init_module(void)
 	int n_created_devices = 0;
 	struct device *device = NULL;
 
-	pr_info("sysdig-probe: driver loading\n");
+	pr_info("driver loading\n");
+
+	ret = get_tracepoint_handles();
+	if (ret < 0)
+		goto init_module_err;
 
 	/*
 	 * Initialize the ring buffers array
@@ -1055,11 +1221,11 @@ int init_module(void)
 	}
 
 	for_each_online_cpu(cpu) {
-		pr_info("sysdig-probe: initializing ring buffer for CPU %u\n", cpu);
+		pr_info("initializing ring buffer for CPU %u\n", cpu);
 
 		alloc_ring_buffer(&per_cpu(g_ring_buffers, cpu));
 		if (per_cpu(g_ring_buffers, cpu) == NULL) {
-			pr_err("sysdig-probe: can't initialize the ring buffer for CPU %u\n", cpu);
+			pr_err("can't initialize the ring buffer for CPU %u\n", cpu);
 			ret = -ENOMEM;
 			goto init_module_err;
 		}
@@ -1067,28 +1233,31 @@ int init_module(void)
 
 	/*
 	 * Initialize the user I/O
+	 * ( + 1 for sysdig-events)
 	 */
-	acrret = alloc_chrdev_region(&dev, 0, num_cpus, PPM_DEVICE_NAME);
+	acrret = alloc_chrdev_region(&dev, 0, num_cpus + 1, PPM_DEVICE_NAME);
 	if (acrret < 0) {
-		pr_err("sysdig-probe: could not allocate major number for %s\n", PPM_DEVICE_NAME);
+		pr_err("could not allocate major number for %s\n", PPM_DEVICE_NAME);
 		ret = -ENOMEM;
 		goto init_module_err;
 	}
 
 	g_ppm_class = class_create(THIS_MODULE, PPM_DEVICE_NAME);
 	if (IS_ERR(g_ppm_class)) {
-		pr_err("sysdig-probe: can't allocate device class\n");
+		pr_err("can't allocate device class\n");
 		ret = -EFAULT;
 		goto init_module_err;
 	}
 
+	g_ppm_class->devnode = ppm_devnode;
+
 	g_ppm_major = MAJOR(dev);
 	g_ppm_numdevs = num_cpus;
 	g_ppm_devs = kmalloc(g_ppm_numdevs * sizeof(struct ppm_device), GFP_KERNEL);
 	if (!g_ppm_devs) {
+		pr_err("can't allocate devices\n");
 		ret = -ENOMEM;
 		goto init_module_err;
-		pr_err("sysdig-probe: can't allocate devices\n");
 	}
 
 	/*
@@ -1099,7 +1268,7 @@ int init_module(void)
 		g_ppm_devs[j].dev = MKDEV(g_ppm_major, j);
 
 		if (cdev_add(&g_ppm_devs[j].cdev, g_ppm_devs[j].dev, 1) < 0) {
-			pr_err("sysdig-probe: could not allocate chrdev for %s\n", PPM_DEVICE_NAME);
+			pr_err("could not allocate chrdev for %s\n", PPM_DEVICE_NAME);
 			ret = -EFAULT;
 			goto init_module_err;
 		}
@@ -1111,7 +1280,7 @@ int init_module(void)
 				       j);
 
 		if (IS_ERR(device)) {
-			pr_err("sysdig-probe: error creating the device for  %s\n", PPM_DEVICE_NAME);
+			pr_err("error creating the device for  %s\n", PPM_DEVICE_NAME);
 			cdev_del(&g_ppm_devs[j].cdev);
 			ret = -EFAULT;
 			goto init_module_err;
@@ -1123,10 +1292,37 @@ int init_module(void)
 
 	/* create_proc_read_entry(PPM_DEVICE_NAME, 0, NULL, ppm_read_proc, NULL); */
 
+	g_ppe_cdev = cdev_alloc();
+	if (g_ppe_cdev == NULL) {
+		pr_err("error allocating the device %s\n", PPE_DEVICE_NAME);
+		ret = -ENOMEM;
+		goto init_module_err;
+	}
+
+	cdev_init(g_ppe_cdev, &g_ppe_fops);
+
+	if (cdev_add(g_ppe_cdev, MKDEV(g_ppm_major, g_ppm_numdevs), 1) < 0) {
+		pr_err("could not allocate chrdev for %s\n", PPE_DEVICE_NAME);
+		ret = -EFAULT;
+		goto init_module_err;
+	}
+
+	g_ppe_dev = device_create(g_ppm_class, NULL,
+			MKDEV(g_ppm_major, g_ppm_numdevs),
+			NULL, /* no additional data */
+			PPE_DEVICE_NAME);
+
+	if (IS_ERR(g_ppe_dev)) {
+		pr_err("error creating the device for  %s\n", PPE_DEVICE_NAME);
+		ret = -EFAULT;
+		goto init_module_err;
+	}
+
 	/*
 	 * All ok. Final initalizations.
 	 */
-	atomic_set(&g_open_count, 0);
+	g_open_count = 0;
+	g_tracepoint_registered = false;
 	g_dropping_mode = 0;
 
 	return 0;
@@ -1138,6 +1334,12 @@ init_module_err:
 
 	/* remove_proc_entry(PPM_DEVICE_NAME, NULL); */
 
+	if (g_ppe_dev != NULL)
+		device_destroy(g_ppm_class, MKDEV(g_ppm_major, g_ppm_numdevs));
+
+	if (g_ppe_cdev != NULL)
+		cdev_del(g_ppe_cdev);
+
 	for (j = 0; j < n_created_devices; ++j) {
 		device_destroy(g_ppm_class, g_ppm_devs[j].dev);
 		cdev_del(&g_ppm_devs[j].cdev);
@@ -1149,18 +1351,17 @@ init_module_err:
 	if (acrret == 0)
 		unregister_chrdev_region(dev, g_ppm_numdevs);
 
-	if (g_ppm_devs)
-		kfree(g_ppm_devs);
+	kfree(g_ppm_devs);
 
 	return ret;
 }
 
-void cleanup_module(void)
+void sysdig_exit(void)
 {
 	int j;
 	int cpu;
 
-	pr_info("sysdig-probe: driver unloading\n");
+	pr_info("driver unloading\n");
 
 	/* remove_proc_entry(PPM_DEVICE_NAME, NULL); */
 
@@ -1172,10 +1373,23 @@ void cleanup_module(void)
 		cdev_del(&g_ppm_devs[j].cdev);
 	}
 
+	if (g_ppe_dev != NULL)
+		device_destroy(g_ppm_class, MKDEV(g_ppm_major, g_ppm_numdevs));
+
+	if (g_ppe_cdev != NULL)
+		cdev_del(g_ppe_cdev);
+
 	if (g_ppm_class)
 		class_destroy(g_ppm_class);
 
-	unregister_chrdev_region(MKDEV(g_ppm_major, 0), g_ppm_numdevs);
+	/* + 1 for sysdig-events */
+	unregister_chrdev_region(MKDEV(g_ppm_major, 0), g_ppm_numdevs + 1);
 
 	kfree(g_ppm_devs);
+
+	tracepoint_synchronize_unregister();
 }
+
+module_init(sysdig_init);
+module_exit(sysdig_exit);
+module_param(verbose, bool, 0);
