@@ -275,9 +275,27 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 	if(target_res <= 0)
 	{
 		//
-		// This is normal and happens with kernel threads, which we aren't interested in
+		// No exe. This either
+		//  - a kernel thread (if there is no cmdline). In that case we skip it.
+		//  - a process that has been containerized or has some weird thing going on. In that case
+		//    we accept it.
 		//
-		return SCAP_SUCCESS;
+		snprintf(filename, sizeof(filename), "%scmdline", dir_name);
+		f = fopen(filename, "r");
+		if(f == NULL)
+		{
+			return SCAP_SUCCESS;
+		}
+
+		if(fgets(line, SCAP_MAX_PATH_SIZE, f) == NULL)
+		{
+			fclose(f);
+			return SCAP_SUCCESS;
+		}
+		else
+		{
+			fclose(f);
+		}
 	}
 
 	target_name[target_res] = 0;
@@ -385,6 +403,35 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 		fclose(f);
 	}
 
+	//
+	// Gather the environment
+	//
+	snprintf(filename, sizeof(filename), "%senviron", dir_name);
+
+	f = fopen(filename, "r");
+	if(f == NULL)
+	{
+		snprintf(error, SCAP_LASTERR_SIZE, "can't open %s", filename);
+		free(tinfo);
+		return SCAP_FAILURE;
+	}
+	else
+	{
+		filesize = fread(line, 1, sizeof(line), f);
+		line[filesize - 1] = 0;
+
+		tinfo->env_len = filesize;
+		if(tinfo->env_len > SCAP_MAX_ENV_SIZE)
+		{
+			tinfo->env_len = SCAP_MAX_ENV_SIZE;
+		}
+
+		memcpy(tinfo->env, line, tinfo->env_len);
+		tinfo->env[SCAP_MAX_ENV_SIZE - 1] = 0;
+
+		fclose(f);
+	}
+
 	//
 	// set the current working directory of the process
 	//
